/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.cosmos.rm.smlif.internal.actions;

import java.io.File;

import org.eclipse.core.resources.IResource;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLCommonUtil;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLMessages;
import org.eclipse.cosmos.rm.smlif.internal.export.ExportException;
import org.eclipse.cosmos.rm.smlif.internal.export.ExportToSMLIF;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;


public class ExportToSMLIFWizardPage extends WizardPage {
	
	protected String smlResourcePath;
	protected String smlResourceName;
	protected Text smlifText;
	protected Button browse;
	protected FileDialog dialog = null;
	protected IStructuredSelection selection;

	public ExportToSMLIFWizardPage(String pageName, IStructuredSelection selection) {
		super(pageName);
		setTitle(pageName); //NON-NLS-1
		//setDescription("Create an SML-IF file from the SML files available in the selected project."); //NON-NLS-1
		
		if (selection == null || selection.isEmpty()) {
			smlResourcePath = "";
			smlResourceName = "";
		}
		
		IResource sel = (IResource) selection.getFirstElement();
		if (sel.getType() == IResource.FILE) {
			smlResourcePath = sel.getParent().getLocation().toOSString();
		} else {
			smlResourcePath = sel.getLocation().toOSString();
		}
		smlResourceName = sel.getName();
		setSelection(selection);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.WizardNewFileCreationPage#createAdvancedControls(org.eclipse.swt.widgets.Composite)
	 */	
	public void createControl(Composite parent) {
		Composite fileSelectionArea = new Composite(parent, SWT.NONE);
		GridData fileSelectionData = new GridData(GridData.GRAB_HORIZONTAL
				| GridData.FILL_HORIZONTAL);
		fileSelectionArea.setLayoutData(fileSelectionData);

		GridLayout fileSelectionLayout = new GridLayout();
		fileSelectionLayout.numColumns = 3;
		fileSelectionLayout.makeColumnsEqualWidth = false;
		fileSelectionLayout.marginWidth = 0;
		fileSelectionLayout.marginHeight = 0;
		fileSelectionLayout.marginLeft = 2;
		fileSelectionLayout.marginRight = 2;
		fileSelectionArea.setLayout(fileSelectionLayout);
		
		Label label = new Label(fileSelectionArea, SWT.NULL);
		label.setText("&SML-IF File Path:");
		
		smlifText = new Text(fileSelectionArea, SWT.BORDER); //NON-NLS-1 //NON-NLS-2
		smlifText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				setPageComplete(smlifText.getText().length() > 0);
			}
		});
		smlifText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		browse = new Button(fileSelectionArea, SWT.PUSH);
		browse.setText("&Browse...");
		browse.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				if (dialog == null) {
					dialog = new FileDialog(getShell());
					dialog.setText("Save SML-IF file as");
					dialog.setFileName("smlif.xml");
					dialog.setFilterPath(smlResourcePath);
					dialog.setFilterExtensions(new String[] { "*.xml", "*" });
				}

				String name = dialog.open();
				if (name != null) {
					smlifText.setText(name);
				}
			}
		});
						
		setControl(fileSelectionArea);//(IResource) selection.getFirstElement()
		String fileNameSuggestion = (((IResource) selection.getFirstElement()).getType() == IResource.FILE) ? "smlif.xml" : smlResourceName+".xml";
		// TODO make better file name suggestion here?
		smlifText.setText(new StringBuffer(smlResourcePath).append(File.separator)
				.append(fileNameSuggestion).toString());
		setPageComplete(smlifText.getText().length() > 0);
		GridData data = new GridData();
		data.widthHint = 400;
		smlifText.setLayoutData(data);
		Dialog.applyDialogFont(fileSelectionArea);
	}
	
	public boolean createSMLIFFile() {
		ExportToSMLIF smlif = new ExportToSMLIF();
		boolean result = false;
		try {
			result = smlif.buildSMLIFFileFromFolder(smlResourcePath, smlifText.getText());
		} catch (ExportException ex) {
			SMLCommonUtil.openErrorWithDetail(SMLMessages.exportErrorTitle, ex.getMessage(), ex);
		}
		return result;
	}

	 public boolean createSMLIFFileFromModelUnits() {
		// TODO change to support multi selection
		ExportToSMLIF smlif = new ExportToSMLIF();
		boolean result = false;
		try {
			result = smlif.buildSMLIFFileFromModelUnits(selection.iterator(), smlifText.getText());
		} catch (ExportException ex) {
			SMLCommonUtil.openErrorWithDetail(SMLMessages.exportErrorTitle, ex.getMessage(), ex);
		}
		return result;
	}

	public void setSelection(IStructuredSelection selection) {
		this.selection = selection;
	}
}
