/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.cosmos.rm.smlif.internal.actions;

import java.io.File;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.Platform;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLCommonUtil;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLMessages;
import org.eclipse.cosmos.rm.smlif.internal.importing.ImportException;
import org.eclipse.cosmos.rm.smlif.internal.importing.ImportFromSMLIF;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;


public class ImportFromSMLIFWizardPage extends WizardPage {
	
	protected String selectedSMLIFFilePath;
	protected String selectedSMLIFFileName;
	protected Text modelDirText;
	protected Text smlifFileText;
	protected DirectoryDialog dirDialog = null;
	protected FileDialog fileDialog = null;
	protected IStructuredSelection selection;

	public ImportFromSMLIFWizardPage(String pageName, IStructuredSelection selection) {
		super(pageName);
		setTitle(pageName); //NON-NLS-1
		//setDescription("Create an SML-IF file from the SML files available in the selected project."); //NON-NLS-1
		
		if (selection == null || selection.isEmpty()) {
			selectedSMLIFFilePath = "";
			selectedSMLIFFileName = "";
		} else {
			IResource sel = (IResource) selection.getFirstElement();
			selectedSMLIFFilePath = sel.getParent().getLocation().toOSString();
			selectedSMLIFFileName = sel.getName();
		}
		setSelection(selection);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.WizardNewFileCreationPage#createAdvancedControls(org.eclipse.swt.widgets.Composite)
	 */	
	public void createControl(Composite parent) {
		Composite topComposite = new Composite(parent, SWT.NONE); 

		GridLayout topCompositeLayout = new GridLayout();
		topCompositeLayout.numColumns = 1;
		topCompositeLayout.makeColumnsEqualWidth = false;
		topCompositeLayout.marginWidth = 0;
		topCompositeLayout.marginHeight = 0;
		topCompositeLayout.marginLeft = 3;
		topCompositeLayout.marginRight = 2;
		
		topComposite.setLayout(topCompositeLayout);
		
		Group pathsGroup = new Group(topComposite, SWT.NONE);
		pathsGroup.setText("Paths");
		GridData pathsGroupData = new GridData(GridData.GRAB_HORIZONTAL
				| GridData.FILL_HORIZONTAL);
		pathsGroup.setLayoutData(pathsGroupData);

		GridLayout pathsGroupLayout = new GridLayout();
		pathsGroupLayout.numColumns = 3;
		pathsGroupLayout.makeColumnsEqualWidth = false;
		pathsGroupLayout.marginWidth = 3;
		pathsGroupLayout.marginHeight = 3;
		pathsGroupLayout.marginLeft = 3;
		pathsGroupLayout.marginRight = 3;
		pathsGroup.setLayout(pathsGroupLayout);
		
		GridData radioData = new GridData(GridData.GRAB_HORIZONTAL);
		radioData.grabExcessHorizontalSpace = true;
		radioData.horizontalSpan = 3;
		Button importFromWorkspaceButton = new Button(pathsGroup, SWT.RADIO);
		importFromWorkspaceButton.setText("Import from workspace");
		importFromWorkspaceButton.setLayoutData(radioData);
		importFromWorkspaceButton.setSelection(true);
		Button importFromFileSystemButton = new Button(pathsGroup, SWT.RADIO);
		importFromFileSystemButton.setText("Import from file system");
		importFromFileSystemButton.setLayoutData(radioData);
		
		Label smlifFileLabel = new Label(pathsGroup, SWT.NULL);
		smlifFileLabel.setText("&SML-IF Source File:");

		smlifFileText = new Text(pathsGroup, SWT.BORDER); //NON-NLS-1 //NON-NLS-2
		smlifFileText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				updatePageComplete();
			}

		});
		smlifFileText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Button browseForSmlifFileButton = new Button(pathsGroup, SWT.PUSH);
		browseForSmlifFileButton.setText("&Browse...");
		browseForSmlifFileButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
//				if (dialog == null) {
//					dialog = new DirectoryDialog(getShell());
//					dialog.setText("Import model to");
//					dialog.setMessage("Model directory");
//					//dialog.setFilterPath(targetModelLocation);
//				}

				if (fileDialog == null) {
					fileDialog = new FileDialog(getShell());
					fileDialog.setText("SML-IF File");
					if (smlifFileText.getText().length() > 0) {
						fileDialog.setFileName(new File(smlifFileText.getText()).getName());
						fileDialog.setFilterPath(new File(smlifFileText.getText()).getParentFile().getAbsolutePath());
					}
//					fileDialog.setFileName(selectedSMLIFFileName);
//					fileDialog.setFilterPath(selectedSMLIFFilePath);
					fileDialog.setFilterExtensions(new String[] { "*.xml", "*" });
				}
				String name = fileDialog.open();
				if (name != null) {
					smlifFileText.setText(name);
				}
			}
		});

		Label modelDirLabel = new Label(pathsGroup, SWT.NULL);
		modelDirLabel.setText("&Destination:");
		
		modelDirText = new Text(pathsGroup, SWT.BORDER); //NON-NLS-1 //NON-NLS-2
		modelDirText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				setPageComplete(modelDirText.getText().length() > 0);
			}
		});
		modelDirText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Button browseForDirectoryButton = new Button(pathsGroup, SWT.PUSH);
		browseForDirectoryButton.setText("&Browse...");
		browseForDirectoryButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				if (dirDialog == null) {
					dirDialog = new DirectoryDialog(getShell());
					dirDialog.setText("Import model to");
					dirDialog.setMessage("Model directory");
					//dialog.setFilterPath(targetModelLocation);
				}

				String name = dirDialog.open();
				if (name != null) {
					modelDirText.setText(name);
				}
			}
		});

		Button genMetadataFileButton = new Button(topComposite, SWT.CHECK);
		genMetadataFileButton.setText("Generate a meta-data file if needed");
		genMetadataFileButton.setSelection(true);
		Button overwriteFilesWithoutWarningButton = new Button(topComposite, SWT.CHECK);
		overwriteFilesWithoutWarningButton.setText("Overwrite files without warning");
		Button createFolderStructureButton = new Button(topComposite, SWT.CHECK);
		createFolderStructureButton.setText("Create necessary folder structure");
		createFolderStructureButton.setSelection(true);
		
		setControl(topComposite);//(IResource) selection.getFirstElement()
		String dirNameSuggestion = ((selection == null) || (selection.size() == 0)) ?  Platform.getUserLocation().toString() : ((IFile) selection.getFirstElement()).getProject().getLocation().toOSString();
		modelDirText.setText(dirNameSuggestion);
		smlifFileText.setText(selectedSMLIFFilePath + File.separatorChar + selectedSMLIFFileName);
		updatePageComplete();
//		GridData data = new GridData();
//		data.widthHint = 400;
//		modelDirText.setLayoutData(data);
//		smlifFileText.setLayoutData(data);
		Dialog.applyDialogFont(pathsGroup);
	}
	
	public boolean createModel() {
		ImportFromSMLIF importer = new ImportFromSMLIF();
		boolean result = false;
		try {
			result = importer.createModelFromSMLIFFile(smlifFileText.getText(), modelDirText.getText());
		} catch (ImportException ex) {
			SMLCommonUtil.openErrorWithDetail(SMLMessages.importErrorTitle, ex.getMessage(), ex);
		}
		return result;
	}

	public void setSelection(IStructuredSelection selection) {
		this.selection = selection;
	}

	private void updatePageComplete() {
		setPageComplete((smlifFileText.getText().length() > 0) && (modelDirText.getText().length() > 0));
	}
}
