/**********************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.smlif.internal.actions;

import java.util.Iterator;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.cosmos.rm.smlif.SMLPlugin;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLCommonUtil;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLMessages;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.eclipse.cosmos.rm.validation.internal.core.IValidator;
import org.eclipse.cosmos.rm.validation.internal.core.MainValidator;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.PartInitException;

/**
 * Represents the validate context menu action that is applicable to the project,
 * folder, or files selected.
 * 
 * @author Ali Mehregani
 */
public class ValidateResourceAction extends AbstractAction
{
	/**
	 * The validator
	 */
	private MainValidator validator;
	
	/**
	 * Constructor
	 */
	public ValidateResourceAction()
	{
		validator = new MainValidator();
	}
	
	
	/**
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	public void run(IAction action)
	{
		if (noValidSelection())
			return;
		
		Map attributes = validator.getAttributes();
		
		
		/* We need to determine if the validator needs to be run against a single SML-IF
		 * file or a set of SML model units */
		IStructuredSelection selection = getSelection();
		IResource firstElement = null;
		boolean validateIFDocument = selection.size() == 1 && 
				(firstElement = (IResource)selection.getFirstElement()) instanceof IFile &&
				SMLCommonUtil.identifyDocumentType((IFile)firstElement) == ISMLConstants.TYPE_SMLIF;
		
		
		/* Validate a single SML-IF document */
		if (validateIFDocument)	
		{
			attributes.put(IValidator.ATTRIBUTE_INPUT_TYPE, IValidator.VALUE_SML_IF);
			attributes.put(IValidator.ATTRIBUTE_INSTANCE, firstElement.getRawLocation().toString());						
		}
		
		/* Validate multiple SML units */
		else
		{
			StringBuffer filePaths = new StringBuffer();
			for (Iterator selectedElements = getSelection().iterator(); selectedElements.hasNext();)
			{
				IResource resource = (IResource)selectedElements.next();
				if (filePaths.length() > 0)
					filePaths.append(",");
				filePaths.append(resource.getFullPath());
			}
					
			attributes.put(IValidator.ATTRIBUTE_INPUT_TYPE, IValidator.VALUE_SML_UNIT);
			attributes.put(IValidator.ATTRIBUTE_INSTANCE, filePaths.toString());			
		}

		attributes.put(IValidator.ATTRIBUTE_ENV, IValidator.VALUE_ENV_ECLIPSE);		
		boolean validation = validator.start();		
		
		/* Display a dialog notifying the user whether or not there were errors */
		if (validation)
		{
			MessageDialog.openInformation(SMLPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(), 
					SMLMessages.confirmNoErrorTitle, SMLMessages.confirmNoError);
		}
		else
		{
			/* Display the problems view */
			try
			{
				SMLPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage().showView(IPageLayout.ID_PROBLEM_VIEW);
			} catch (PartInitException e)
			{
				MessageDialog.openError(SMLPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(), 
					SMLMessages.failedValidationTitle, SMLMessages.failedValidation);
			}
		}
	}
}
