/**********************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.smlif.internal.importing;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Iterator;

import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;
import org.eclipse.cosmos.rm.smlif.internal.common.SMLMessages;
import org.eclipse.cosmos.rm.validation.internal.common.ISMLConstants;
import org.eclipse.osgi.util.NLS;
import org.xml.sax.SAXException;

/**
 * Import model structure from SML-IF file.
 * 
 * @author David Whiteman
 */
public class ImportFromSMLIF {

	public boolean createModelFromSMLIFFile(String smlifFileName, String targetDirName) throws ImportException {
		if (smlifFileName == null) {
			throw new ImportException(SMLMessages.importErrorNullSmlifFileName);
		}
		if (targetDirName == null) {
			throw new ImportException(SMLMessages.importErrorNullTargetDirName);
		}
		SMLIFFileHandler handler = new SMLIFFileHandler();
		try {
//			System.out.println("parsing "+smlifFileName);
			SAXParserFactory saxParserFactory = SAXParserFactory.newInstance();
			saxParserFactory.setNamespaceAware(true);

			saxParserFactory.setFeature(ISMLConstants.SAX_PARSER_FEATURE_NAMESPACES, true);
			saxParserFactory.setFeature(ISMLConstants.SAX_PARSER_FEATURE_NAMESPACE_PREFIXES, true);

			SAXParser parser = saxParserFactory.newSAXParser();
			parser.parse(smlifFileName, handler);
			for (Iterator i = handler.getModelUnits().iterator(); i.hasNext();) {
				ModelUnit modelUnit = (ModelUnit) i.next();
				String targetFileName = modelUnit.getFileName();
//				IFile file = null;
//				if (targetFileName.startsWith("/")) {
//					// workspace relative path
//					file = ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(targetFileName));
//				} else {
//					// here we're building a path on the file system
//					targetFileName = (new File(smlifFileName).getParent()) + File.separatorChar + targetFileName;
//					file = ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(new Path(targetFileName));
//				}
				File smlifFile = new File(targetDirName + File.separatorChar + handler.getModelLocation() + targetFileName);
				if ((!smlifFile.getParentFile().exists()) && !smlifFile.getParentFile().mkdirs()) {
					throw new ImportException(NLS.bind(SMLMessages.importErrorCouldNotCreateDirectory, smlifFile.getParent().toString()), null);
				}
//				System.out.println("writing "+smlifFile);
				BufferedWriter writer = null;
				try {
					writer = new BufferedWriter(new OutputStreamWriter(
							new FileOutputStream(smlifFile), "UTF-8"));
					writer.write(modelUnit.getFileContents().toString());
				} catch (IOException exc) {
					//getLog().log(new Status(Status.INFO, PLUGIN_ID, Status.OK, msg, e));
					throw new ImportException(NLS.bind(SMLMessages.importErrorCouldNotWriteFile, smlifFile.toString()), exc);
				} finally {
					if (writer != null) {
						writer.close();
					}
				}
				
				IFile file = ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(new Path(smlifFile.getAbsolutePath()));
				if (file != null) {
					file.refreshLocal(IResource.DEPTH_INFINITE, null);
				}
			}
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (ImportException e) {
			// Need to re-throw this so it is passed to callers
			// specialized handling (e.g. error dialog)
			throw e;
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (FactoryConfigurationError e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (CoreException e) {
			e.printStackTrace();
		}
		return true;
	}
}
