package org.eclipse.cosmos.rm.validation.internal.databuilders;

import java.io.IOException;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;

import junit.framework.Test;

import org.eclipse.cosmos.rm.validation.internal.common.UtilityTester;
import org.eclipse.hyades.test.common.junit.DefaultTestArbiter;
import org.eclipse.hyades.test.common.junit.HyadesTestCase;
import org.eclipse.hyades.test.common.junit.HyadesTestSuite;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

/**
 * Generated code for the test suite <b>TestDocumentDOMBuilder</b> located at
 * <i>/org.eclipse.cosmos.rm.validation.tests/junits/TestDocumentDOMBuilder.testsuite</i>.
 */
public class TestDocumentDOMBuilder extends HyadesTestCase
{
	/**
	 * Constructor for TestDocumentDOMBuilder.
	 * @param name
	 */
	public TestDocumentDOMBuilder(String name)
	{
		super(name);
	}

	/**
	 * Returns the JUnit test suite that implements the <b>TestDocumentDOMBuilder</b> definition.
	 */
	public static Test suite()
	{
		HyadesTestSuite testDocumentDOMBuilder = new HyadesTestSuite("TestDocumentDOMBuilder");
		testDocumentDOMBuilder.setArbiter(DefaultTestArbiter.INSTANCE).setId("CD6DD3B2A4B2FE778F5553608BEF11DB");
		testDocumentDOMBuilder.addTest(new TestDocumentDOMBuilder("testGeneration").setId("CD6DD3B2A4B2FE77918616108BEF11DB").setTestInvocationId("CD6DD3B2A4B2FE77AC3F5D408BEF11DB"));
		testDocumentDOMBuilder.addTest(new TestDocumentDOMBuilder("testInvalidCylcesGeneration").setId("CA88AFF1D7B860F59003A3E095F711DB").setTestInvocationId("CA88AFF1D7B860F5A17016E095F711DB"));
		return testDocumentDOMBuilder;
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception
	{
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception
	{
	}

	private Map parseDocument(String path, String[] expectedAliases, String[] expectedNodes) throws ParserConfigurationException, SAXException, IOException
	{		
		DocumentDOMBuilder documentDOMBuilder = new DocumentDOMBuilder();		
		UtilityTester.parseDocument(this.getClass().getClassLoader().getResourceAsStream(path), documentDOMBuilder);
		Map domDocuments = (Map)documentDOMBuilder.getDataStructure();
		for (int i = 0; i < expectedAliases.length; i++)
		{
			Node node = (Node) domDocuments.get(expectedAliases[i]);
			assertNotNull("The alias: " + expectedAliases[i] + " was not found", node);
			assertEquals("The node name was expected to be Student", expectedNodes[i], node.getNodeName());
		}
		
		return domDocuments;
	}
	
	/**
	 * testGeneration
	 * @throws Exception
	 */
	public void testGeneration() throws Exception
	{
		final String[] EXPECTED_ALIASES =  new String[] {"Student1", "Student2", "Student3", "Student4"};
		final String[] EXPECTED_NODE_NAME = new String[] {"Student", "Student", "Student", "Student"};
		Map domDocuments = parseDocument ("test-resources/deref/Multiple.xml", EXPECTED_ALIASES, EXPECTED_NODE_NAME);
		List list = (List)domDocuments.get(DocumentDOMBuilder.NO_ALIAS);
		assertNotNull("The orphan list is null", list);
		assertEquals("The size of the orphan list is expected to be 1", 1, list.size());
		assertEquals("The node name of the orphan node is expected to be Universtiy", "University", ((Node)list.get(0)).getNodeName());
	}

	/**
	 * testInvalidCylcesGeneration
	 * @throws Exception
	 */
	public void testInvalidCylcesGeneration() throws Exception
	{
		final String[] EXPECTED_ALIASES =  new String[] {"Course1", "Course2", "Course3"};
		final String[] EXPECTED_NODE_NAME = new String[] {"Course", "Course", "Course"};
		parseDocument ("test-resources/acyclic/InValidCycles.xml", EXPECTED_ALIASES, EXPECTED_NODE_NAME);
	}

}
