/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.common;

/**
 * Defines constants that are specific to the SML/SML-IF documents
 * 
 * @author Ali Mehregani
 */
public class ISMLConstants
{
	/**
	 * Elements
	 */
	public static final String DOCUMENT_ELEMENT = "document";
	public static final String ALIAS_ELEMENT = "alias";
	public static final String ALIASES_ELEMENT = "aliases";	
	public static final String URI_ELEMENT = "uri";	
	public static final String GENICS_ELEMENT = "genics";
	public static final String PHENICS_ELEMENT = "phenics";
    public static final String COMPLEXTYPE_ELEMENT = "complexType";
    public static final String ELEMENT_ELEMENT = "element";
    public static final String EXTENSION_ELEMENT = "extension";    
    public static final String DATA_ELEMENT = "data";
    public static final String SCHEMA_ELEMENT = "schema";
    public static final String KEY_ELEMENT = "key";
    public static final String KEY_REF_ELEMENT = "keyref";
    public static final String UNIQUE_ELEMENT = "unique";
    public static final String SML_SELECTOR = "selector";
    public static final String SML_FIELD = "field";
    public static final String IDENTITY_ELEMENT = "identity";
    public static final String NAME_ELEMENT = "name";
    public static final String DESCRIPTION_ELEMENT = "description";
    public static final String BASE_URI_ELEMENT = "baseURI";
    public static final String RULE_ELEMENT = "rule";
    public static final String ANNOTATION_ELEMENT = "annotation";
    public static final String APP_INFO_ELEMENT = "appinfo";    
    public static final String SELECTOR_ELEMENT = "selector";
    public static final String FIELD_ELEMENT = "field";
    public static final String NS_ELEMENT = "ns";
    public static final String RULEBINDING_ELEMENT = "ruleBinding";
	public static final String LOCATOR_ELEMENT = "locator";    
	public static final String DOC_INFO_ELEMENT = "docinfo";
	public static final String MODEL_ELEMENT = "model";
	public static final String DISPLAY_NAME_ELEMENT = "displayName";
	
    /**
     * Data builders interested in fragments that precede the genic/phenic
     * section should use this constant
     */
    public static final byte START_PHASE=0x02;
    
    /**
     * Indicates that the entire document is being 
     * processed
     */
    public static final byte UNKNOWN_PHASE=0x03;
    /**
     * Indicates that the genic section of the document is being 
     * processed
     */
    public static final byte GENICS_PHASE=0x01;
    
    /**
     * Indicates that the phenic section of the document is being 
     * processed 
     */
    public static final byte PHENICS_PHASE=0x10;
    
    /**
     * Data builder interested to be notified while the genic or phenic
     * section is being processed should use this constant.
     */
    public static final byte GENICS_PHENICS_PHASE=0x00;
	
	/**
	 * Attributes
	 */
	public static final String ACYCLIC_ATTRIBUTE = "acyclic";
	public static final String BASE_ATTRIBUTE = "base";	
	public static final String NAME_ATTRIBUTE = "name";	
    public static final String TYPE_ATTRIBUTE ="type";
    public static final String SUBSTITUTIONGROUP_ATTRIBUTE = "substitutionGroup";
    public static final String TARGETELEMENT_ATTRIBUTE="targetElement";
    public static final String TARGET_NAMESPACE_ATTRIBUTE="targetNamespace";
    public static final String TARGETTYPE_ATTRIBUTE="targetType";
    public static final String TARGET_NAMESPACE ="targetNamespace";
    public static final String XPATH_ATTRIBUTE ="xpath";    
    public static final String URI_ATTRIBUTE ="uri";    
    public static final String TARGETREQUIRED_ATTRIBUTE="targetRequired";        
    public static final String XML_NS_ATTRIBUTE = "xmlns";
    public static final String CONTEXT_ATTRIBUTE = "context";
    public static final String REFER_ATTRIBUTE = "refer";
    public static final String PREFIX_ATTRIBUTE = "prefix";
    public static final String DOCUMENTALIAS_ATTRIBUTE = "documentAlias";
    public static final String RULEALIAS_ATTRIBUTE = "ruleAlias";
	public static final String HREF_ATTRIBUTE = "href";    
           
	/**
	 * Attribute values
	 */
	public static final String REF_TYPE_VALUE = "refType";
	
	/**
	 * Signifies that the sml:uri value is null
	 */
	public static final String SML_URI_VALUE_NULL="sml.uri.value.null";
	  
	 
	/**
	 * XPath Extension functions
	 */
	public static final String DEREF_FN = "deref";
	
	/**
	 * Context paramter that is used to specify the rule context
     */
	public static final String CONTEXTPARAM = "$contextParam"; 
	
	/**
	 * Delimintator to signify type key
     */
	public static final String TYPEDELIM = "$typeDelim"; 	
	
	/**
	 * Used to identify genic documents
	 */
	public static final int TYPE_GENIC = 0;
	
	/**
	 * Used to identify phenic documents
	 */
	public static final int TYPE_PHENIC = 1;
		
	/**
	 * Used to identify sml-if document types
	 */
	public static final int TYPE_SMLIF = 2;
	
	/**
	 * Used to identify an unknown content type 
	 */
	public static final int TYPE_UNKNOWN = 3;	
	
	/**
	 * Target namespaces
	 */
	public static final String SML_URI = "http://schemas.serviceml.org/sml/2006/10";
	public static final String SCHEMATRON_URI = "http://purl.oclc.org/dsdl/schematron";	
	public static final String SMLIF_URI = "http://schemas.serviceml.org/smlif/2006/10";
	public static final String SCHEMA_URI = "http://www.w3.org/2001/XMLSchema";
	public static final String XSL_URI = "http://www.w3.org/1999/XSL/Transform";	
	public static final String SML_FN_URI = "http://schemas.serviceml.org/sml/function/2006/07";
	public static final String SML_FN_URI2 = "http://schemas.serviceml.org/smlfn/query/2006/07";	
	public static final String XLINK_URI = "http://www.w3.org/1999/xlink";	
	
	/**
	 * SAX parser constants
	 */
	public static final String SAX_PARSER_PROPERTY_SCHEMA_SOURCE = "http://java.sun.com/xml/jaxp/properties/schemaSource";
	public static final String SAX_PARSER_PROPERTY_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
	public static final String SAX_PARSER_FEATURE_VALIDATION = "http://xml.org/sax/features/validation";
	public static final String SAX_PARSER_FEATURE_NAMESPACE_PREFIXES = "http://xml.org/sax/features/namespace-prefixes";
	public static final String SAX_PARSER_FEATURE_NAMESPACES = "http://xml.org/sax/features/namespaces";
	
}
