/**********************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.cosmos.rm.validation.internal.reference;

import org.eclipse.cosmos.rm.validation.internal.common.SMLCommonUtil;
import org.eclipse.cosmos.rm.validation.internal.common.SMLMessages;
import org.eclipse.cosmos.rm.validation.internal.databuilders.NamespaceContextBuilder;
import org.eclipse.cosmos.rm.validation.internal.databuilders.NamespaceContextBuilder.MappedNamespaceContext;
import org.eclipse.osgi.util.NLS;


/**
 * The XMLNS scheme is used to evaluate the parenthesized expressions that
 * appears after the keyword xmlns in an XPointer expression.
 * 
 * @author Ali Mehregani
 */
public class XmlnsScheme extends AbstractScheme
{
	/**
	 * Represents the keyword to identify this scheme
	 */
	public static final String SCHEME = "xmlns";


	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.reference.IXScheme#evaluate(java.lang.Object)
	 */
	public Object evaluate(Object context) throws BadExpressionException
	{
		MappedNamespaceContext namespaceContext = (MappedNamespaceContext)SMLCommonUtil.retrieveDataStructure(NamespaceContextBuilder.ID);
		if (namespaceContext == null)
			throw new BadExpressionException (SMLMessages.errorMissingNamespace);
		
		/* Parse the expression */
		String expression = getExpression();
		int equalSignInx = -1;
		if (expression == null || (equalSignInx = expression.indexOf('=')) <= 0)
			throw new BadExpressionException (NLS.bind(SMLMessages.errorBadNSExpression, expression));
		
		String prefix = expression.substring(0, equalSignInx);
		String uri = equalSignInx + 1 >= expression.length() ? null : expression.substring(equalSignInx + 1);
		
		if (prefix == null || (prefix = prefix.trim()).length() <= 0)
		{
			throw new BadExpressionException (NLS.bind(SMLMessages.errorBadPrefix, expression));
		}
		else if (uri == null || (uri = uri.trim()).length() <= 0)
		{
			throw new BadExpressionException (NLS.bind(SMLMessages.errorBadURI, expression));
		}
		
		namespaceContext.addEntry(prefix, uri);
		return context;
	}


	/**
	 * @see org.eclipse.cosmos.rm.validation.internal.reference.IXScheme#getType()
	 */
	public String getType()
	{
		return SCHEME;
	}

}
